"use strict";
/// <reference path="../types/index.d.ts" />
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const module_1 = __importDefault(require("module"));
const semver_1 = __importDefault(require("semver"));
const path_1 = __importDefault(require("path"));
const module_2 = require("./node-internals/module");
const url_1 = require("url");
function hasArgument(func, argName) {
    const argmatch = /^function[^(]*\(([^)]*)\)/.exec(func.toString());
    if (!argmatch)
        return false;
    return argmatch[1].split(/,\s*/).includes(argName);
}
function functionalTest(func, ...args) {
    try {
        return func(...args);
    }
    catch (e) {
        return false;
    }
}
function requireTest(createRequire, requireArg, ...moduleIds) {
    if (!createRequire) {
        return false;
    }
    const requireFunc = functionalTest(createRequire, requireArg);
    if (requireFunc === false)
        return false;
    for (const moduleId of moduleIds) {
        if (functionalTest(requireFunc, moduleId) === false)
            return false;
    }
    return true;
}
function debug(...args) {
    if (process.env.DEBUG_NODE_MODULE_POLYFILL) {
        console.debug(...args);
    }
}
const forced_polyfills = (process.env.FORCE_NODE_MODULE_POLYFILL || "").split(":");
function shouldOverride(prop) {
    return process.env.FORCE_NODE_MODULE_POLYFILL == "*" || forced_polyfills.includes(prop);
}
if (typeof process.env.FORCE_NODE_MODULE_POLYFILL === "undefined" && semver_1.default.gte(process.versions.node, "12.2.0")) {
    debug(`Node version ${process.versions.node} >= 12.2.0, not polyfilling`);
}
else {
    debug(`Node version ${process.versions.node} < 12.2.0, applying polyfills`);
    if (!('path' in module)) {
        debug("overriding module.path");
        Object.defineProperty(module_1.default.prototype, 'path', module_2.getPathDescriptor());
    }
    if (shouldOverride("_resolveLookupPaths") || hasArgument(module_1.default._resolveLookupPaths, "newReturn")) {
        debug("overriding Module._resolveLookupPaths()");
        module_1.default._resolveLookupPaths = module_2.poly_resolveLookupPaths(module_1.default._resolveLookupPaths);
    }
    const packageDir_name = path_1.default.basename(path_1.default.dirname(__dirname));
    if (shouldOverride("_resolveFilename") ||
        !hasArgument(module_1.default._resolveFilename, "options") ||
        functionalTest(module_1.default._resolveFilename, `.${path_1.default.sep}${packageDir_name}${path_1.default.sep}package.json`, module, false, { paths: [path_1.default.resolve(__dirname, "..", "..")] }) !== path_1.default.resolve(__dirname, "..", "package.json")) {
        debug("overriding Module._resolveFilename()");
        module_1.default._resolveFilename = module_2.poly_resolveFilename(module_1.default._resolveFilename);
    }
    if (shouldOverride("createRequireFromPath") ||
        !requireTest(module_1.default.createRequireFromPath, __filename, "./node-internals/module") ||
        !requireTest(module_1.default.createRequireFromPath, path_1.default.join(__dirname, path_1.default.sep), "./node-internals/module")) {
        debug("overriding Module.createRequireFromPath()");
        module_1.default.createRequireFromPath = module_2.createRequireFromPath;
    }
    if (shouldOverride("createRequire") ||
        !requireTest(module_1.default.createRequire, __filename, "./node-internals/module") ||
        !requireTest(module_1.default.createRequire, path_1.default.join(__dirname, path_1.default.sep), "./node-internals/module") ||
        !requireTest(module_1.default.createRequire, new url_1.URL(`file://${__filename}`), "./node-internals/module")) {
        debug("overriding Module.createRequire()");
        module_1.default.createRequire = module_2.createRequire;
    }
}
