"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const module_1 = __importDefault(require("module"));
const assert_1 = __importDefault(require("assert"));
const url_1 = require("url");
const url_2 = require("./url");
const error_1 = require("./error");
const validators_1 = require("./validators");
const isWindows = process.platform === 'win32';
const createRequireError = 'must be a file URL object, file URL string, or ' +
    'absolute path string';
class PoisonModule extends module_1.default {
    constructor() { super('poison-module'); }
    get id() { throw "Poisoned"; }
    set id(_) { } // gets called during constructor
    get filename() { throw "Poisoned"; }
    set filename(_) { } // gets called during constructor
    get paths() { throw "Poisoned"; }
}
function poly_resolveLookupPaths(orig_resolveLookupPaths) {
    function _resolveLookupPaths(request, parent, newReturn) {
        return orig_resolveLookupPaths(request, parent, typeof newReturn === 'undefined' ? true : newReturn);
    }
    return _resolveLookupPaths;
}
exports.poly_resolveLookupPaths = poly_resolveLookupPaths;
function poly_resolveFilename(orig_resolveFilename) {
    if ('_orig_resolveFilename' in orig_resolveFilename) {
        orig_resolveFilename = orig_resolveFilename._orig_resolveFilename;
    }
    assert_1.default(!orig_resolveFilename.toString().includes("PoisonModule"));
    function _resolveFilename(request, parent, isMain, options) {
        try {
            // If {request} is a valid internal module, orig_resolveFilename will return it without
            // ever touching the "parent" we pass it. If it does, the poison pill will send it right back here.
            const simpleResolve = orig_resolveFilename(request, new PoisonModule, false);
            if (simpleResolve === request) {
                return request;
            }
        }
        catch (e) {
            if (e !== "Poisoned") {
                throw e;
            }
        }
        var paths;
        if (typeof options === 'object' && options !== null) {
            if (Array.isArray(options.paths)) {
                const isRelative = request.startsWith('./') ||
                    request.startsWith('../') ||
                    (isWindows && request.startsWith('.\\') ||
                        request.startsWith('..\\'));
                if (isRelative) {
                    paths = options.paths;
                }
                else {
                    const fakeParent = new module_1.default('', null);
                    paths = [];
                    for (var i = 0; i < options.paths.length; i++) {
                        const path = options.paths[i];
                        fakeParent.paths = module_1.default._nodeModulePaths(path);
                        const lookupPaths = module_1.default._resolveLookupPaths(request, fakeParent);
                        for (var j = 0; j < lookupPaths.length; j++) {
                            if (!paths.includes(lookupPaths[j]))
                                paths.push(lookupPaths[j]);
                        }
                    }
                }
            }
            else if (options.paths === undefined) {
                paths = module_1.default._resolveLookupPaths(request, parent);
            }
            else {
                throw new error_1.ERR_INVALID_OPT_VALUE('options.paths', options.paths);
            }
        }
        else {
            paths = module_1.default._resolveLookupPaths(request, parent);
        }
        // Look up the filename first, since that's the cache key.
        const filename = module_1.default._findPath(request, paths, isMain);
        if (!filename) {
            const requireStack = [];
            for (var cursor = parent; cursor; cursor = cursor.parent) {
                requireStack.push(cursor.filename || cursor.id);
            }
            let message = `Cannot find module '${request}'`;
            if (requireStack.length > 0) {
                message = message + '\nRequire stack:\n- ' + requireStack.join('\n- ');
            }
            // eslint-disable-next-line no-restricted-syntax
            var err = new Error(message);
            err.code = 'MODULE_NOT_FOUND';
            err.requireStack = requireStack;
            throw err;
        }
        return filename;
    }
    ;
    _resolveFilename._orig_resolveFilename = orig_resolveFilename;
    return _resolveFilename;
}
exports.poly_resolveFilename = poly_resolveFilename;
function makeRequireFunction(mod, _redirects) {
    const Module = mod.constructor;
    let require;
    require = function require(path) {
        return mod.require(path);
    };
    function resolve(request, options) {
        validators_1.validateString(request, 'request');
        return Module._resolveFilename(request, mod, false, options);
    }
    require.resolve = resolve;
    function paths(request) {
        validators_1.validateString(request, 'request');
        return Module._resolveLookupPaths(request, mod);
    }
    resolve.paths = paths;
    require.main = process.mainModule;
    // Enable support to add extra extension types.
    require.extensions = Module._extensions;
    require.cache = Module._cache;
    return require;
}
function createRequire(filename) {
    let filepath;
    if (filename instanceof url_1.URL ||
        (typeof filename === 'string' && !path_1.default.isAbsolute(filename))) {
        try {
            filepath = url_2.fileURLToPath(filename);
        }
        catch (_a) {
            throw new error_1.ERR_INVALID_ARG_VALUE('filename', filename, createRequireError);
        }
    }
    else if (typeof filename !== 'string') {
        throw new error_1.ERR_INVALID_ARG_VALUE('filename', filename, createRequireError);
    }
    else {
        filepath = filename;
    }
    return module_1.default.createRequireFromPath(filepath);
}
exports.createRequire = createRequire;
function createRequireFromPath(filename) {
    // Allow a directory to be passed as the filename
    const trailingSlash = filename.endsWith('/') || (isWindows && filename.endsWith('\\'));
    const proxyPath = trailingSlash ?
        path_1.default.join(filename, 'noop.js') :
        filename;
    const m = new module_1.default(proxyPath);
    m.filename = proxyPath;
    m.paths = module_1.default._nodeModulePaths(m.path);
    return makeRequireFunction(m, null);
}
exports.createRequireFromPath = createRequireFromPath;
function getPathDescriptor() {
    return {
        get: function getPath() {
            return path_1.default.dirname(this.id);
        },
        set: function setPath(value) {
            Object.defineProperty(this, "path", {
                value: value,
                configurable: true,
                writable: true,
            });
        },
        configurable: true,
    };
}
exports.getPathDescriptor = getPathDescriptor;
